import config.base

import os

class Configure(config.base.Configure):
  def __init__(self, framework):
    config.base.Configure.__init__(self, framework)
    self.headerPrefix = ''
    self.substPrefix  = ''
    self.argDB        = framework.argDB
    self.found        = 0
    self.compilers    = self.framework.require('config.compilers',     self)
    self.libraries    = self.framework.require('config.libraries',     self)
    self.blas         = self.framework.require('PETSc.packages.BLAS', self)
    return

  def configureHelp(self, help):
    import nargs
    help.addArgument('LAPACK', '-with-lapack=<lib>', nargs.Arg(None, None, 'Use LAPACK, optionally indicating the library'))
    return

  def checkLib(self, fullLib):
    '''Checking for the LAPACK library'''
    oldLibs              = self.framework.argDB['LIBS']
    (self.dir, self.lib) = os.path.split(fullLib)
    self.lib             = os.path.splitext(self.lib)[0]
    if self.lib.startswith('lib'): self.lib = self.lib[3:]
    otherLibs            = self.libraries.getLibArgument(self.blas.lib)+' '+self.compilers.flibs
    if self.libraries.check(self.lib, 'dtrtrs', libDir = self.dir, otherLibs = otherLibs, fortranMangle = 1):
      self.found = 1
    self.framework.argDB['LIBS'] = oldLibs
    return self.found

  def setOutput(self):
    if self.found:
      self.addSubstitution('LAPACK_DIR', self.dir)
      libFlag = ''
      if self.dir: libFlag = '-L'+self.dir+' '
      libFlag += self.libraries.getLibArgument(self.lib)
      self.addSubstitution('LAPACK_LIB', libFlag)
    return

  def configure(self):
    try:
      if self.framework.argDB.has_key('with-lapack') and self.framework.argDB['with-lapack']:
        fullLib = self.framework.argDB['with-lapack']
      else:
        fullLib = 'liblapack.a'
      if self.executeTest(self.checkLib, fullLib): return

      petscPath = os.path.abspath(os.path.join(self.argDB['PETSC_DIR'], '..', 'blaslapack', 'lib'))
      fullLib   = os.path.join(petscPath, 'liblapack.a')
      if os.path.exists(fullLib) and self.executeTest(self.checkLib, fullLib): return

      petscPath = os.path.join(petscPath, 'lib'+self.argDB['BOPT'], self.argDB['PETSC_ARCH'])
      fullLib   = os.path.join(petscPath, 'liblapack.a')
      if os.path.exists(fullLib) and self.executeTest(self.checkLib, fullLib): return
    finally:
      self.setOutput()
    return
